/*******************************************/
/*      TS-2Cリモコン 送出コード解析       */
/*******************************************/
//  TS-2Cリモコンの送出コードを調べる
//  入力
//      PB4 12 : 送出開始ストローブ  Hで送信開始
//      PB5 13 : データパルス入力 HLHLパルス　0.5mS、1mS間隔
//  出力
//      PB3 11 : 送出データ確認

/*****  ライブラリ      ****/
#include  "wiring_private.h"      // sbi,cbi命令用
// http://www.arduino.cc/en/Tutorial/LiquidCrystal
 
/*****  アンサインドの変数定義    *****/
typedef unsigned char       uchar;
typedef unsigned int        uint;
typedef unsigned long       ulong;
// typedef uint8_t          byte;
// typedef uint16_t         word;
typedef uint32_t            lword;

/*****   マクロ      *****/
//      配列のﾃﾞｰﾀ数を返すﾏｸﾛ
#define    DIMSIZ(a)    (sizeof(a)/sizeof(*a))

//  入力信号マクロ
#define  INP_PLS   (digitalRead(13))      // パルス入力
#define  INP_STB   (digitalRead(12))      // ストローブ入力
//  出力信号
#define  CHK_H     (sbi(PORTB,PORTB3))    // PB3(11)
#define  CHK_L     (cbi(PORTB,PORTB3))

/*****    測定データ       *****/
byte mes_data[512];         // 8bitバッファ
                            // 先頭のビットはMSB
#define MES_SIZ     (DIMSIZ(mes_data) * 8)  // 記録データ数
word mes_cnt;           // 測定データ数
                        // 1/8してmes_dataのアドレス
                        // 0～7がビット位置 0がbit7

/**************************/
/*    シリアル入出力      */
/**************************/
/*****  1文字出力       *****/
void tx(char c)
{
    Serial.write(c);
}

/*****  書式付シリアル出力     *****/
void txprintf(const char *s, ...)
{
va_list vp;
char bff[40];     // ﾊﾞｯﾌｧを確保
    va_start(vp,s);
    vsprintf(bff,s,vp);
    Serial.print(bff);
    va_end(vp);
}

/*****  シリアル受信チェック  *****/
//  データがあれば1でリターン
byte rxx(void)
{
byte r = 0;
    r = Serial.available();   // 受信データあり?
    return r;   
}

/*****  シリアル受信    *****/
//  8bit データを持ってリターン
char rx(void)
{
char c = 0;
    c = Serial.read();      // データ受信
    return c;   
}

/**************************/
/*    測定実行            */
/**************************/
//  microsで読み出す時間
ulong   tm_us;                  // 1us

/*****  パルス幅で1/0を書き込む   *****/
//  750usで判定
void mespls(ulong us)
{
byte d;
word a;
word b;
    if(mes_cnt < MES_SIZ){      // データ数ok
        a = mes_cnt / 8;        // アドレス
        b = mes_cnt % 8;        // ビット位置 0～7
//      if(b == 0)      mes_data[a]  = 0;       // ゼロクリア
        if(us > 750){
            d = _BV(7 - b);         // MSBから
            mes_data[a] |= d;       // 1/0
        }
        mes_cnt++;              // データ数+1
    }
}


/*****  実行状態      *****/
byte j_exc;                 // 実行区分
#define J_DATATX    5       // 結果知り

/*****  開始メッセージ   *****/
void jstart(void)
{
    txprintf("\r\n#Start (%d)\r\n", MES_SIZ);
    CHK_L;                      // チェック信号
    mes_cnt = 0;                // 測定データ数=0
    memset(mes_data, 0, sizeof(mes_data));    // メモリクリア
    j_exc++;
}

/*****  STBのHを待つ    *****/
void jstbhi(void)
{
    if(INP_STB){                // STB=Hになった
//      Serial.println("#STB H");
        j_exc++;
    }
    else{                       // Lが継続
        if(rxx()){
            rx();
            j_exc = 0;
        }
    }
}

/*****  最初のPLSのHを待つ *****/
//  送信開始を検出
void jstbplshi(void)
{
    if(INP_PLS){                // PLS=Hになった
        CHK_H;                  // チェック信号
//      Serial.println("#PLS H");
        tm_us = micros();       // 時間記録
        j_exc++;
    }
    else{                       // PLS = Lの時
        if(INP_STB == 0){       // STB = Lならおわり
            j_exc = J_DATATX;
        }
    }
}

/*****  PLSのLを待つ    *****/
void jplslo(void)
{
ulong t0,t1;
    if(INP_PLS == 0){           // PLS=Lになった
        CHK_L;                  // チェック信号
        t0 = micros();          // 現時間
        t1 = t0 - tm_us;
        mespls(t1);             // 0/1 ?
        tm_us = micros();       // 時間記録
        j_exc++;                // H待ちに
    }       
    else{                       // PLS = Hの時
        if(INP_STB == 0){       // STB = Lならおわり
            j_exc = J_DATATX;
        }
    }
}

/*****  PLSのHを待つ *****/
void jplshi(void)
{
ulong t0,t1;
    if(INP_PLS){                // PLS=Hになった
        CHK_H;                  // チェック信号
        t0 = micros();          // 現時間
        t1 = t0 - tm_us;
        mespls(t1);             // 0/1 ?
        tm_us = micros();       // 時間記録
        j_exc--;                // L待ちに
    }       
    else{                       // PLS = Lの時
        if(INP_STB == 0){       // STB = Lならおわり
            j_exc = J_DATATX;
        }
    }
}

/*****  測定データ送出 *****/
void jdatatx(void)
{
word i, j;
word a;
byte b, d, d0, d1;
    CHK_L;                              // チェック信号
    txprintf("#data=%d\r\n", mes_cnt);
    j = 0;
    for(i = 0; i < mes_cnt; i++){
        a = i / 8;                      // アドレス
        b = 7 - (i % 8);                // bit位置 MSBから
        if(mes_data[a] & _BV(b))    tx('1');
        else                        tx('.');
        if(b == 0){
            j++;
            if(j >= 8){
                Serial.println();
                j = 0;
            }
            else{
                tx(' ');
            }       
        }
    }
    Serial.println();
//  16進数で
    j = 0;
    for(i = 0; i < mes_cnt; i+=16){
        a = i / 8;                      // アドレス
        d0 = mes_data[a];               // 2bitを1bitにまとめる
        d1 = mes_data[a+1];
        d = 0;
        if(d0 & 0xc0)   d |= 0x80;      // MSBから
        if(d0 & 0x30)   d |= 0x40;
        if(d0 & 0x0c)   d |= 0x20;
        if(d0 & 0x03)   d |= 0x10;
        if(d1 & 0xc0)   d |= 0x08;
        if(d1 & 0x30)   d |= 0x04;
        if(d1 & 0x0c)   d |= 0x02;
        if(d1 & 0x03)   d |= 0x01;
        txprintf("%02X", d);
        j++;
        if(j >= 8){                     // 1行8バイト
            Serial.println();
            j = 0;
        }
        else{
            tx(' ');
        }
    }
    j_exc = 0;
}

/*****  判定実行        *****/
void (*jexc[])(void)={
    jstart,             // 0 開始メッセージ
    jstbhi,             // 1 STB=Hを待つ
    jstbplshi,          // 2 最初のPLS=H
    jplslo,             // 3 PLS=L
    jplshi,             // 4 PLS=H
    jdatatx,            // 5 結果出力
}; 

/************************************/
/*      セットアップ／ループ        */
/************************************/
/*****  SET UP      *****/
void setup(void)
{
    pinMode(13, INPUT);     // PLS 送出コードパルス
    pinMode(12, INPUT);     // STB 送信開始
    pinMode(11, OUTPUT);    // CHK チェック出力
    Serial.begin(9600);
}

/*****  ループ     *****/
void loop(void)
{
    while(1){    
        jexc[j_exc]();
    }
}

/*===== end of <TS_2C_PLS.INO>      =====*/
